/*
    This file is part of the UNF library / toolset
    Copyright (C) 2000 Steffen Kaiser

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* $RCSfile: READ_UNF.C $
   $Locker: ska $	$Name:  $	$State: Exp $

	Read in an UNF file and dump it into directory "out".
	Resolves different radixs, $include, $chain_to etc.


 */
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#if (UINT_MAX <= 0xffffU)
  /* quick (and dirty) check ... assuming 16bits as DOS */
# include <dos.h>
# if defined(__TURBOC__)
#  include <dir.h>
# else
#  include <direct.h>
# endif
# define my_mkdir   mkdir
#else
# if defined(__UNIX__) || defined(__DJGPP__)    /* posix style */
#  include <sys/stat.h>
#  include <sys/types.h>
#  define my_mkdir(p)  mkdir(p, S_IRWXU)
# else                                          /* windows style */
#  include <direct.h>
#  define my_mkdir  _mkdir
# endif
#endif

#include "../unf.h"
#if 1
#include "../version.h"
#endif

const char outdir[] = "out";

#ifndef lint
static char const rcsid[] = 
	"$Id: READ_UNF.C 1.2 2000/07/02 04:51:06 ska Exp ska $";
#endif

#if 1
int main(int argc, char **argv)
{
    struct UNF_DATA *d;
    char *progname;
    int optinv = 0;
    int opth = 0;
    char *optd = NULL;
    char *src = NULL, *dest = NULL;
    char *p;
    
    progname = "read_unf";
    
    while(argc > 1) {
        --argc;
        p = *++argv;
        if (*p == '-' && p[1] != '-') switch(p[1]) {
        case '?': case 'H': case 'h':
            opth = 1;
            break;
        case 'D': case 'd':
            optd = p + 2;
            break;
        }
        else {
            if (!src) src = p;
            else if (!dest) dest = p;
        }
    }
    
    if (opth || (!src && !dest)) {
        printf("Usage: %s [-doutdir] src_unffile\n"
               "       %s src_unffile dest_unffile\n"
               "Read an UNF file and dump it with resolving different radixs, $include, \n"
               "$chain_to etc.\n"
               "(suitable for kernel 20%02d%s)\n",
               progname,
               progname,
               REVISION_SEQ,
#if DBCS
               " DBCS"
#else
               ""
#endif
        );
        return 1;
    }
    
    if (!optd)
        optd = (char *)outdir;
    if (*optd == '\0')
        optd = ".";
    
    if (!dest) {
        if (strcmp(optd, ".") != 0 && strcmp(optd, "..") != 0) {
            if (my_mkdir(optd) < 0) {
                fprintf(stderr, "Failed to create a subdirectory (errno=%d): %s\n", (int)errno, optd);
                return 1;
            }
        }
    }
    if((d = unf_load(src, UNF_DEFAULT, UNF_DEFAULT, (UNF_Error_Fct *)unf_error)) == NULL) {
        fprintf(stderr, "Failed to load UNF file: %s\n", src);
        return 33;
    }
    if (dest)
        unf_dump0(!0, d, dest);
    else
        unf_dump(d, optd);
    fprintf(stderr, "Dumped re-encoded UNF file (%03u:%03u): %s"
                  , d->country, d->codepage, src);
    if (dest)
        fprintf(stderr, " to %s", dest);
    fprintf(stderr, "\n");

    return 0;
}
#else
main(int argc, char **argv)
{	struct UNF_DATA *d;

	if(argc != 2) {
		puts("Useage: read_unf UNF_file");
		return 1;
	}

    if (my_mkdir(outdir) < 0) {
        fprintf(stderr, "Failed to make a subdirectory (errno=%d): %s\n", (int)errno, outdir);
        return 1;
    }
	if((d = unf_load(argv[1], UNF_DEFAULT, UNF_DEFAULT, (UNF_Error_Fct *)unf_error)) == NULL) {
		fprintf(stderr, "Failed to load UNF file: %s\n", argv[1]);
		return 33;
	}
	unf_dump(d, outdir);
	fprintf(stderr, "Dumped re-encoded UNF file (%03u:%03u): %s\n"
	 , d->country, d->codepage, argv[1]);
	return 0;
}
#endif
